#define RING_BUFFER_NOREAD_AVAILABLE 1

#define MAX_FILTER_SIZE 0x10000

typedef struct RingBuffer 
{
    int id;
    float *data;
    unsigned long samples_per_block;
    unsigned long number_blocks;
    unsigned long block_size;
    unsigned long index_mask;
    unsigned long block_write_index;
    unsigned long block_read_index;
    unsigned long num_blocks_available;
    
} RingBuffer;

typedef struct DualRingBuffer {
    int nchannels;
    RingBuffer *buffer_1;
    RingBuffer *buffer_2;
} DualRingBuffer;

typedef struct RingBufferArray {
    int nbuf;
    RingBuffer **buffers;
    float **data;
} RingBufferArray;

int ring_buffer_init(RingBuffer *buffer, unsigned long number_blocks_exponent, unsigned long samples_per_block);
int ring_buffer_delete(RingBuffer *buffer);
int ring_buffer_write(RingBuffer *buffer, float *data);
int ring_buffer_read(RingBuffer *buffer, float **data);
int ring_buffer_full(RingBuffer *buffer);
int ring_buffer_empty(RingBuffer *buffer);
int ring_buffer_available(RingBuffer *buffer, int *available);
int ring_buffer_write_sample(RingBuffer *buffer, float data, int index);
int ring_buffer_write_next_block(RingBuffer *buffer);

