#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "portaudio.h"
#include "table_dds.h"

typedef struct {
    float *table_ch1;
    float *table_ch2;
    unsigned long increment_ch1, increment_ch2;
    unsigned long accum_ch1;
    unsigned long accum_ch2;
} tableData;

PaStream *stream;
tableData table_data;


static int table_dds_callback( const void *inputBuffer, void *outputBuffer,
                            unsigned long framesPerBuffer,
                            const PaStreamCallbackTimeInfo* timeInfo,
                            PaStreamCallbackFlags statusFlags,
                            void *userData )
{
    tableData *data = (tableData*)userData;
    float *out = (float*)outputBuffer;
    unsigned long i;

    (void) timeInfo; /* Prevent unused variable warnings. */
    (void) statusFlags;
    (void) inputBuffer;
    for( i=0; i<framesPerBuffer; i++ ) {
           data->accum_ch1 += data->increment_ch1;
           data->accum_ch1 &= ACCUM_MASK;
           *out++ = data->table_ch1[data->accum_ch1 >> ACCUM_SHIFT];
           data->accum_ch2 += data->increment_ch2;
           data->accum_ch2 &= ACCUM_MASK;
           *out++ = data->table_ch2[data->accum_ch2 >> ACCUM_SHIFT];
    }
    return paContinue;
}

static void StreamFinished( void* userData )
{
       
}


int table_dds_start(int sample_rate, int frames_per_buffer, float *table_ch1, float *table_ch2, double frequency_ch1, double frequency_ch2) {
    
    PaStreamParameters outputParameters;
    PaError err; 
    
    table_data.accum_ch1 = 0;
    table_data.accum_ch2 = 0;
    table_data.increment_ch1 = (unsigned long)floor(pow(2,ACCUM_NBITS)*frequency_ch1/((double)sample_rate));
    table_data.increment_ch2 = (unsigned long)floor(pow(2,ACCUM_NBITS)*frequency_ch2/((double)sample_rate));
    table_data.table_ch1 = table_ch1;
    table_data.table_ch2 = table_ch2;
    
    err = Pa_Initialize();
    if( err != paNoError ) {Pa_Terminate(); fprintf( stderr, "Error : %s\n", Pa_GetErrorText( err ) ); return err;}
    outputParameters.device = Pa_GetDefaultOutputDevice(); /* default output device */
    if (outputParameters.device == paNoDevice) {
      fprintf(stderr,"Error: No default output device.\n");
      Pa_Terminate();
      return 1; 
    }
    outputParameters.channelCount = 2;   
    outputParameters.sampleFormat = paFloat32; 
    outputParameters.suggestedLatency = Pa_GetDeviceInfo( outputParameters.device )->defaultLowOutputLatency;
    outputParameters.hostApiSpecificStreamInfo = NULL;
    err = Pa_OpenStream(
              &stream,
              NULL, /* no input */
              &outputParameters,
              sample_rate,
              frames_per_buffer,
              0, // clip ON  
              table_dds_callback,
              &table_data );
    if( err != paNoError ) {Pa_Terminate(); fprintf( stderr, "Error : %s\n", Pa_GetErrorText( err ) ); return err;}
    err = Pa_SetStreamFinishedCallback( stream, &StreamFinished );
    if( err != paNoError ) {Pa_Terminate(); fprintf( stderr, "Error : %s\n", Pa_GetErrorText( err ) ); return err;}
    err = Pa_StartStream( stream );
    if( err != paNoError ) {Pa_Terminate(); fprintf( stderr, "Error : %s\n", Pa_GetErrorText( err ) ); return err;}
    
    return -1;
}

int table_dds_stop(int seconds) {
       PaError err;
       Pa_Sleep( seconds * 1000 );
    err = Pa_StopStream( stream );
    if( err != paNoError ) {Pa_Terminate(); fprintf( stderr, "Error : %s\n", Pa_GetErrorText( err ) ); return err;}
    err = Pa_CloseStream( stream );
    if( err != paNoError ) {Pa_Terminate(); fprintf( stderr, "Error : %s\n", Pa_GetErrorText( err ) ); return err;}
    Pa_Terminate();
    return -1;
}

