/*
Copyright Frédéric LEGRAND, Lycée Chateaubriand, Rennes (2015) 

Ce logiciel est un programme informatique servant d'interface entre
python et les convertisseurs N/A Eurosmart SysamPCI et SysamSP5 

Ce logiciel est régi par la licence CeCILL soumise au droit français et
respectant les principes de diffusion des logiciels libres. Vous pouvez
utiliser, modifier et/ou redistribuer ce programme sous les conditions
de la licence CeCILL telle que diffusée par le CEA, le CNRS et l'INRIA 
sur le site "http://www.cecill.info".

En contrepartie de l'accessibilité au code source et des droits de copie,
de modification et de redistribution accordés par cette licence, il n'est
offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
seule une responsabilité restreinte pèse sur l'auteur du programme,  le
titulaire des droits patrimoniaux et les concédants successifs.

A cet égard  l'attention de l'utilisateur est attirée sur les risques
associés au chargement,  à l'utilisation,  à la modification et/ou au
développement et à la reproduction du logiciel par l'utilisateur étant 
donné sa spécificité de logiciel libre, qui peut le rendre complexe à 
manipuler et qui le réserve donc à des développeurs et des professionnels
avertis possédant  des  connaissances  informatiques approfondies.  Les
utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
logiciel à leurs besoins dans des conditions permettant d'assurer la
sécurité de leurs systèmes et ou de leurs données et, plus généralement, 
à l'utiliser et l'exploiter dans les mêmes conditions de sécurité. 

Le fait que vous puissiez accéder à cet en-tête signifie que vous avez 
pris connaissance de la licence CeCILL, et que vous en avez accepté les
termes.
*/
#define NPY_NO_DEPRECATED_API NPY_1_7_API_VERSION

#include <Python.h> 
#include <numpy/npy_common.h>
#include <numpy/ndarrayobject.h>
#include "pysysam.h"
#include "sysam.h"

#ifdef AUDIO
#include "table_dds.h"
#include "ring_buffer.h"
#include "output_stream.h"
#endif

Handle handle;  

#define ARG_ERROR(e) if (e) {PyErr_SetString(PyExc_ValueError,"bad arguments"); return NULL;}
#define SYSAM_PCI_NO if (handle.type==PCI) {PyErr_SetString(PyExc_ValueError,"fonction non implementee sur Sysam PCI");return NULL;}

static PyObject* _get_error_msg() {
    return PyUnicode_FromStringAndSize(handle.errormsg,(Py_ssize_t)MSGSIZE);  
}


static PyObject* py_can_ouvrir(PyObject *self, PyObject *args) {
       int sys;
       PyObject *error_msg;
       ARG_ERROR(!PyArg_ParseTuple(args,"i",&sys));
       handle.present = 0;
       handle.tabEntrees = NULL;
       handle.tabTemps = NULL;
       handle.tabSortieS1 = NULL;
       handle.tabSortieS2 = NULL;
       handle.nombreEA = 0; 
       handle.nbpointsEA = 0;
       if (sys==1) {
            handle.type = SP5;
            handle.can_ouvrir = sp5_ouvrir;
            handle.can_fermer = sp5_fermer;
            handle.can_reset = sp5_reset;
            handle.can_config_entrees = sp5_config_entrees;
            handle.can_config_trigger = sp5_config_trigger;
            handle.can_config_echantillon = sp5_config_echantillon;
            handle.can_config_echantillon_permanent = sp5_config_echantillon_permanent;
            handle.can_acquerir = sp5_acquerir;
            handle.can_acquerir_permanent = sp5_acquerir_permanent;
            handle.can_acquerir_permanent_sansfin = sp5_acquerir_permanent_sansfin;
            handle.can_stopper_acquisition = sp5_stopper_acquisition;
            handle.can_acquerir_avec_sorties = sp5_acquerir_avec_sorties;
            handle.can_config_sortie = sp5_config_sortie;
            handle.can_declencher_sorties = sp5_declencher_sorties;
            handle.can_stopper_sorties = sp5_stopper_sorties;
            handle.can_activer_lecture = sp5_activer_lecture;
            handle.can_lire = sp5_lire;
            handle.can_desactiver_lecture = sp5_desactiver_lecture;
            handle.can_ecrire = sp5_ecrire;
            handle.can_lancer = sp5_lancer;
            handle.can_lancer_permanent = sp5_lancer_permanent;
            handle.can_lancer_permanent_sansfin = sp5_lancer_permanent_sansfin;
            handle.can_lancer_avec_sorties = sp5_lancer_avec_sorties;
            handle.can_portC_config = sp5_portC_config;
            handle.can_portC_ecrire = sp5_portC_ecrire;
            handle.can_portC_lire = sp5_portC_lire;
            handle.can_portB_config = sp5_portB_config;
            handle.can_portB_ecrire = sp5_portB_ecrire;
            handle.can_portB_lire = sp5_portB_lire;
            handle.can_config_filtre = sp5_config_filtre;
            handle.can_config_compteur = sp5_config_compteur;
            handle.can_compteur = sp5_compteur;
            handle.can_lire_compteur = sp5_lire_compteur;
            handle.can_config_chrono = sp5_config_chrono;
            handle.can_chrono = sp5_chrono;
            handle.can_lire_chrono = sp5_lire_chrono;
            handle.can_afficher_calibrage = sp5_afficher_calibrage;
       }
       else if (sys==2) {
            handle.type = PCI;
            handle.can_ouvrir = pci_ouvrir;
            handle.can_fermer = pci_fermer;
            handle.can_config_entrees = pci_config_entrees;
            handle.can_config_trigger = pci_config_trigger;
            handle.can_config_echantillon = pci_config_echantillon;
            handle.can_acquerir = pci_acquerir;
            handle.can_acquerir_avec_sorties = pci_acquerir_avec_sorties;
            handle.can_config_sortie = pci_config_sortie;
            handle.can_declencher_sorties = pci_declencher_sorties;
            handle.can_stopper_sorties = pci_stopper_sorties;
            handle.can_activer_lecture = pci_activer_lecture;
            handle.can_lire = pci_lire;
            handle.can_desactiver_lecture = pci_desactiver_lecture;
            handle.can_ecrire = pci_ecrire;
            handle.can_lancer = pci_lancer;
            handle.can_lancer_avec_sorties = pci_lancer_avec_sorties;   
       }
       else {
            PyErr_SetString(PyExc_ValueError,"CAN inconnu");
            return NULL;
       }
       if (handle.can_ouvrir(&handle)==0) {
            error_msg = _get_error_msg();
            PyErr_SetObject(PyExc_ValueError,error_msg);
            return NULL;
       }
       Py_RETURN_NONE;
}

void fermer_can() {
    handle.can_fermer(&handle);
    if (handle.tabEntrees!=NULL) free(handle.tabEntrees);
    if (handle.tabTemps!=NULL) free(handle.tabTemps);
    if (handle.tabSortieS1!=NULL) free(handle.tabSortieS1);
    if (handle.tabSortieS2!=NULL) free(handle.tabSortieS2);
    if (handle.tabEntreesFiltrees!=NULL) free(handle.tabEntreesFiltrees);
    if (handle.filtreA!=NULL) free(handle.filtreA);
    if (handle.filtreB!=NULL) free(handle.filtreB);
}

static PyObject* py_can_fermer(PyObject *self, PyObject *args) {
      fermer_can();
      Py_RETURN_NONE;
}

static PyObject* py_can_reset(PyObject *self, PyObject *args) {
      if (handle.type==SP5) handle.can_reset(&handle);
      Py_RETURN_NONE;
}



void PyIntegerList2Array(PyObject *list, int **array, int *size) {
    int i;
    int v;
    *size = PyList_Size(list);
    *array = (int*)malloc(sizeof(int)*(*size));
    for (i=0; i<*size; i++) {
        v = (int)PyLong_AsLong(PyList_GetItem(list,i));
        (*array)[i] =v;
    }
}
void PyFloatList2Array(PyObject *list, double **array, int *size) {
    int i;
    double v;
    *size = PyList_Size(list);
    *array = (double*)malloc(sizeof(double)*(*size));
    for (i=0; i<*size; i++) {
        v = (double)PyFloat_AsDouble(PyList_GetItem(list,i));
        (*array)[i] = v;
    }
}

int NumpyList2Array(PyArrayObject *np_array, double **array, int *size) {
    int type,i;
    double v;
    void *data;
    type = PyArray_TYPE(np_array);
    if ((type!=NPY_DOUBLE)&&(type!=NPY_FLOAT)) return 0;
    if (PyArray_NDIM(np_array)!=1) return 0;
    *size = PyArray_DIM(np_array,0);
    *array = (double*)malloc(sizeof(double)*(*size));
    for (i=0; i<*size; i++) {
         data = PyArray_GETPTR1(np_array,i);
         switch(type) {
         case NPY_DOUBLE:
             v = (double)(((double*)data)[0]);
             (*array)[i] = v;
              break;
         case NPY_FLOAT:
             v = (double)(((float*)data)[0]);
             (*array)[i] = v;
             break;
         
         }
    }
    return 1;
}

static PyObject* py_can_config_entrees(PyObject *self, PyObject *args) {
    PyObject *voies_array,*calibres_array,*diff_array;
    int nbVoies, nbCalibres, nbDiff; 
    int *voies,*diff;
    double *calibres;
    PyObject *error_msg;
    ARG_ERROR(!PyArg_ParseTuple(args,"OOO",&voies_array,&calibres_array,&diff_array))
    PyIntegerList2Array(voies_array,&voies,&nbVoies);
    PyFloatList2Array(calibres_array,&calibres,&nbCalibres);
    PyIntegerList2Array(diff_array,&diff,&nbDiff);
    if (handle.can_config_entrees(&handle,voies,nbVoies,calibres,nbCalibres,diff,nbDiff)==0) {
        fermer_can();
        error_msg = _get_error_msg();
        PyErr_SetObject(PyExc_ValueError,error_msg);
        return NULL;    
    }
    free(voies); free(calibres); free(diff);
    Py_RETURN_NONE;
}

static PyObject* py_can_activer_lecture(PyObject *self, PyObject *args) {
     PyObject *error_msg;
     PyObject *voies_array;
     int latch;
     int *voies,nbVoies;
     ARG_ERROR(!PyArg_ParseTuple(args,"Oi",&voies_array,&latch))
     PyIntegerList2Array(voies_array,&voies,&nbVoies);
     if (handle.can_activer_lecture(&handle,voies,nbVoies,latch)==0) {
        fermer_can();
        error_msg = _get_error_msg();
        PyErr_SetObject(PyExc_ValueError,error_msg);
        return NULL;   
     }
     free(voies);
     Py_RETURN_NONE;
}

static PyObject* py_can_desactiver_lecture(PyObject *self, PyObject *args) {
    PyObject *error_msg;
    if (handle.can_desactiver_lecture(&handle)==0) {
        fermer_can();
        error_msg = _get_error_msg();
        PyErr_SetObject(PyExc_ValueError,error_msg);
        return NULL;  
    }
    Py_RETURN_NONE;  
}

static PyObject* py_can_config_trigger(PyObject *self, PyObject *args) {
    PyObject *error_msg;
    int externe,voie,montant,pretrigger,pretriggerSouple,hysteresis;
    double seuil;
    ARG_ERROR(!PyArg_ParseTuple(args,"iidiiii",&externe,&voie,&seuil,&montant,&pretrigger,&pretriggerSouple,&hysteresis))
    if (handle.can_config_trigger(&handle,externe,voie,seuil,montant,pretrigger,pretriggerSouple,hysteresis)==0) {
        fermer_can();
        error_msg = _get_error_msg();
        PyErr_SetObject(PyExc_ValueError,error_msg);
        return NULL;
    }
    Py_RETURN_NONE;
}

static PyObject* py_can_config_echantillon(PyObject *self, PyObject *args) {
    PyObject *error_msg;
    float temps;
    int nbpoints;
    ARG_ERROR(!PyArg_ParseTuple(args,"fi",&temps,&nbpoints))
    if (handle.can_config_echantillon(&handle,temps,nbpoints)==0) {
        fermer_can();
       error_msg = _get_error_msg();
        PyErr_SetObject(PyExc_ValueError,error_msg);
        return NULL;   
    }
    Py_RETURN_NONE;
}

static PyObject* py_can_config_quantification(PyObject *self, PyObject *args) {
    int quantification;
    ARG_ERROR(!PyArg_ParseTuple(args,"i",&quantification))
    if ((quantification <= 12)&&(quantification>=1)) 
        handle.masqueQ = (0x0FFF >> (12-quantification))<<(12-quantification);
    Py_RETURN_NONE;
}

static PyObject* py_can_config_echantillon_permanent(PyObject *self, PyObject *args) {
    PyObject *error_msg;
    float temps;
    int nbpoints;
    ARG_ERROR(!PyArg_ParseTuple(args,"fi",&temps,&nbpoints))
    SYSAM_PCI_NO
    if (handle.can_config_echantillon_permanent(&handle,temps,nbpoints)==0) {
        fermer_can();
       error_msg = _get_error_msg();
        PyErr_SetObject(PyExc_ValueError,error_msg);
        return NULL;   
    }
    Py_RETURN_NONE;
}

static PyObject* py_can_acquerir(PyObject *self, PyObject *args) {
    PyObject *error_msg;
    if (handle.can_acquerir(&handle)==0) {
       fermer_can();
        error_msg = _get_error_msg();
        PyErr_SetObject(PyExc_ValueError,error_msg);
        return NULL;  
    }
    Py_RETURN_NONE;
}

static PyObject* py_can_acquerir_permanent(PyObject *self, PyObject *args) {
    PyObject *error_msg;
    SYSAM_PCI_NO
     if (handle.can_acquerir_permanent(&handle)==0) {
            fermer_can();
            error_msg = _get_error_msg();
            PyErr_SetObject(PyExc_ValueError,error_msg);
            return NULL;  
        }
    Py_RETURN_NONE;
}



static PyObject* py_can_lancer(PyObject *self, PyObject *args) {
    PyObject *error_msg;
    if (handle.can_lancer(&handle)==0) {
        fermer_can();
        error_msg = _get_error_msg();
        PyErr_SetObject(PyExc_ValueError,error_msg);
        return NULL;  
    }
    Py_RETURN_NONE;   
}

static PyObject* py_can_lancer_permanent(PyObject *self, PyObject *args) {
   PyObject *error_msg;
    int repetition;
    ARG_ERROR(!PyArg_ParseTuple(args,"i",&repetition))
    SYSAM_PCI_NO
    if (repetition==0) {
        if (handle.can_lancer_permanent(&handle)==0) {
            fermer_can();
            error_msg = _get_error_msg();
            PyErr_SetObject(PyExc_ValueError,error_msg);
            return NULL;  
        }
    }
    else {
        if (handle.can_lancer_permanent_sansfin(&handle)==0) {
            fermer_can();
            error_msg = _get_error_msg();
            PyErr_SetObject(PyExc_ValueError,error_msg);
            return NULL;  
        }
    }
    Py_RETURN_NONE;
}

static PyObject* py_can_stopper_acquisition(PyObject *self, PyObject *args) {
      PyObject *error_msg;
      if (handle.can_stopper_acquisition(&handle)==0) {
            fermer_can();
            error_msg = _get_error_msg();
            PyErr_SetObject(PyExc_ValueError,error_msg);
            return NULL;
      }
      Py_RETURN_NONE;
}

static PyObject* py_can_entrees(PyObject *self, PyObject *args) {
    npy_intp dims[2], c[2];
    PyObject *numpy_array;
    double *data;
    long p;
    char v;
    int nbpoints;
    int reduction;
    ARG_ERROR(!PyArg_ParseTuple(args,"i",&reduction))
    if (handle.tabEntrees==NULL) {
        fermer_can();
        PyErr_SetString(PyExc_ValueError,"aucune acquisition effectuée");
        return NULL;   
    }
    if (handle.nbpointsEAdemande < handle.nbpointsEA) nbpoints = handle.nbpointsEAdemande;
    else nbpoints = handle.nbpointsEA;
    nbpoints = nbpoints/reduction;
    dims[0] = handle.nombreEA;
    dims[1] = nbpoints;
    numpy_array = PyArray_SimpleNew(2,dims,NPY_DOUBLE);
    c[0] = 0; c[1] = 0;
    data = (double*)PyArray_GetPtr((PyArrayObject*)numpy_array,c);
    for (p=0; p < nbpoints; p++) {
            for (v=0; v < handle.nombreEA; v++) {
                data[v*nbpoints+p] = handle.tabEntrees[p*reduction*handle.nombreEA+v];
            }
    }
    return Py_BuildValue("O",numpy_array);
}

static PyObject* py_can_entrees_filtrees(PyObject *self, PyObject *args) {
    npy_intp dims[2], c[2];
    PyObject *numpy_array;
    double *data;
    long p;
    char v;
    int nbpoints;
    int reduction;
    ARG_ERROR(!PyArg_ParseTuple(args,"i",&reduction))
    SYSAM_PCI_NO
    if (handle.tabEntrees==NULL) {
        fermer_can();
        PyErr_SetString(PyExc_ValueError,"aucune acquisition effectuée");
        return NULL;   
    }
    if (handle.nbpointsEAdemande < handle.nbpointsEA) nbpoints = handle.nbpointsEAdemande;
    else nbpoints = handle.nbpointsEA;
    nbpoints = nbpoints/reduction;
    dims[0] = handle.nombreEA;
    dims[1] = nbpoints;
    numpy_array = PyArray_SimpleNew(2,dims,NPY_DOUBLE);
    c[0] = 0; c[1] = 0;
    data = (double*)PyArray_GetPtr((PyArrayObject*)numpy_array,c);
    for (p=0; p < nbpoints; p++) {
            for (v=0; v < handle.nombreEA; v++) {
                data[v*nbpoints+p] = handle.tabEntreesFiltrees[p*reduction*handle.nombreEA+v];
            }
    }
    return Py_BuildValue("O",numpy_array);
}


static PyObject* py_can_temps(PyObject *self, PyObject *args) {
    npy_intp dims[2], c[2];
    PyObject *numpy_array;
    double *data;
    long p;
    char v;
    int nbpoints;
    int reduction;
    ARG_ERROR(!PyArg_ParseTuple(args,"i",&reduction))
    if (handle.tabEntrees==NULL) {
        fermer_can();
        PyErr_SetString(PyExc_ValueError,"aucune acquisition effectuée");
        return NULL;   
    }
    if (handle.nbpointsEAdemande < handle.nbpointsEA) nbpoints = handle.nbpointsEAdemande;
    else nbpoints = handle.nbpointsEA;
    nbpoints = nbpoints/reduction;
    dims[0] = handle.nombreEA;
    dims[1] = nbpoints;
    numpy_array = PyArray_SimpleNew(2,dims,NPY_DOUBLE);
    c[0] = 0; c[1] = 0;
    data = (double*)PyArray_GetPtr((PyArrayObject*)numpy_array,c);
    for (p=0; p < nbpoints; p++) {
            for (v=0; v < handle.nombreEA; v++) {
                data[v*nbpoints+p] = handle.tabTemps[p*reduction*handle.nombreEA+v];
            } 
    }
    return Py_BuildValue("O",numpy_array);
}

static PyObject* py_can_nombre_echant(PyObject *self, PyObject *args) {
    return Py_BuildValue("i",handle.nbpointsEAlus);
}

static PyObject* py_can_paquet(PyObject *self, PyObject *args) {
    npy_intp dims[2], c[2];
    PyObject *numpy_array; 
    double *data;
    long p;
    char v;
    int premier;
    int nbpoints;
    int skip,dskip;
    int k;
    int reduction;
    int i;
    
    ARG_ERROR(!PyArg_ParseTuple(args,"ii",&premier,&reduction))
    if (premier<0) premier = 0;
    nbpoints = handle.nbpointsEAlus;
    nbpoints = nbpoints/reduction;
    dims[0] = handle.nombreEA*3;
    dims[1] = nbpoints-premier;
    if ((dims[1]<=0)||(handle.tabEntrees==NULL)) {
           dims[1] = 0;
           numpy_array = PyArray_SimpleNew(2,dims,NPY_DOUBLE);
           return Py_BuildValue("O",numpy_array); 
    }
    numpy_array = PyArray_SimpleNew(2,dims,NPY_DOUBLE);
    c[0] = 0; c[1] = 0;
    data = (double*)PyArray_GetPtr((PyArrayObject*)numpy_array,c);
    skip = handle.nombreEA*dims[1];
    dskip = 2*skip;
    for (p=premier; p < nbpoints; p++) {
        for (v=0; v < handle.nombreEA; v++) {
            k = v*dims[1]+(p-premier);
            i = p*reduction*handle.nombreEA+v;
            data[k] = handle.tabTemps[i];
            data[k+skip] = handle.tabEntrees[i];
            data[k+2*skip] = handle.tabEntreesFiltrees[i];
        }   
    }
    return Py_BuildValue("O",numpy_array);
}

static PyObject* py_can_paquet_filtrees(PyObject *self, PyObject *args) {
    npy_intp dims[2], c[2];
    PyObject *numpy_array; 
    double *data;
    long p;
    char v;
    int premier;
    int nbpoints;
    int reduction;
    int skip;
    int k;
    int i;
    ARG_ERROR(!PyArg_ParseTuple(args,"ii",&premier,&reduction))
    SYSAM_PCI_NO
    if (premier<0) premier = 0;
    nbpoints = handle.nbpointsEAlus;
    nbpoints = nbpoints/reduction;
    dims[0] = handle.nombreEA*2;
    dims[1] = nbpoints-premier;
    if ((dims[1]<=0)||(handle.tabEntrees==NULL)) {
           dims[1] = 0;
           numpy_array = PyArray_SimpleNew(2,dims,NPY_DOUBLE);
           return Py_BuildValue("O",numpy_array); 
    }
    numpy_array = PyArray_SimpleNew(2,dims,NPY_DOUBLE);
    c[0] = 0; c[1] = 0;
    data = (double*)PyArray_GetPtr((PyArrayObject*)numpy_array,c);
    skip = handle.nombreEA*dims[1];
    for (p=premier; p < nbpoints; p++) {
        for (v=0; v < handle.nombreEA; v++) {
            k = v*dims[1]+(p-premier);
            i = p*reduction*handle.nombreEA+v;
            data[k] = handle.tabTemps[i];
            data[k+skip] = handle.tabEntreesFiltrees[i];
            
        }   
    }
    return Py_BuildValue("O",numpy_array);
}

static PyObject* py_can_paquet_circulaire(PyObject *self, PyObject *args) {
    npy_intp dims[2], c[2];
    PyObject *numpy_array; 
    double *data;
    long p;
    char v;
    int nbpoints;
    int reduction;
    int skip,dskip;
    int k;
    int ipts_circ;
    int i;
    
    ARG_ERROR(!PyArg_ParseTuple(args,"i",&reduction))
    SYSAM_PCI_NO
    if (handle.nb_tampons_dispo==0) nbpoints=0;
    else nbpoints = handle.nbpointsEA;
    nbpoints = nbpoints/reduction;
    dims[0] = handle.nombreEA*3;
    dims[1] = nbpoints;
    if ((dims[1]<=0)||(handle.tabEntrees==NULL)) {
           dims[1] = 0;
           numpy_array = PyArray_SimpleNew(2,dims,NPY_DOUBLE);
           return Py_BuildValue("O",numpy_array); 
    }
    numpy_array = PyArray_SimpleNew(2,dims,NPY_DOUBLE);
    c[0] = 0; c[1] = 0;
    data = (double*)PyArray_GetPtr((PyArrayObject*)numpy_array,c);
    skip = handle.nombreEA*dims[1];
    dskip = 2*skip;
    ipts_circ = 0;
    for (p=0; p < nbpoints; p++) {
        for (v=0; v < handle.nombreEA; v++) {
            k = v*dims[1]+p;
            i = handle.pos_tampon_lecture+ipts_circ*handle.nombreEA+v;
            data[k] = handle.tabTemps[i];
            data[k+skip] = handle.tabEntrees[i];
            data[k+dskip] = handle.tabEntreesFiltrees[i];
        }
        ipts_circ += reduction;
        
    }
    handle.pos_tampon_lecture += handle.nbpointsEA*handle.nombreEA;
    if (handle.pos_tampon_lecture== handle.nbpointsEA*handle.nombreEA*NB_TAMPONS) handle.pos_tampon_lecture=0;
    handle.nb_tampons_dispo--;
    return Py_BuildValue("O",numpy_array);
}

static PyObject* py_can_paquet_circulaire_filtrees(PyObject *self, PyObject *args) {
    npy_intp dims[2], c[2];
    PyObject *numpy_array; 
    double *data;
    long p;
    char v;
    int nbpoints;
    int reduction;
    int skip;
    int k;
    int ipts_circ;
    int i;
    ARG_ERROR(!PyArg_ParseTuple(args,"i",&reduction))
    SYSAM_PCI_NO
    if (handle.nb_tampons_dispo==0) nbpoints=0;
    else nbpoints = handle.nbpointsEA;
    nbpoints = nbpoints/reduction;
    dims[0] = handle.nombreEA*2;
    dims[1] = nbpoints;
    if ((dims[1]<=0)||(handle.tabEntrees==NULL)) {
           dims[1] = 0;
           numpy_array = PyArray_SimpleNew(2,dims,NPY_DOUBLE);
           return Py_BuildValue("O",numpy_array); 
    }
    numpy_array = PyArray_SimpleNew(2,dims,NPY_DOUBLE);
    c[0] = 0; c[1] = 0;
    data = (double*)PyArray_GetPtr((PyArrayObject*)numpy_array,c);
    skip = handle.nombreEA*dims[1];
    ipts_circ = 0;
    for (p=0; p < nbpoints; p++) {
        for (v=0; v < handle.nombreEA; v++) {
            k = v*dims[1]+p;
            i = handle.pos_tampon_lecture+ipts_circ*handle.nombreEA+v;
            data[k] = handle.tabTemps[i];
            data[k+skip] = handle.tabEntreesFiltrees[i];
            
        }
        ipts_circ += reduction;
        
    }
    handle.pos_tampon_lecture += handle.nbpointsEA*handle.nombreEA;
    if (handle.pos_tampon_lecture== handle.nbpointsEA*handle.nombreEA*NB_TAMPONS) handle.pos_tampon_lecture=0;
    handle.nb_tampons_dispo--;
    return Py_BuildValue("O",numpy_array);
}

static PyObject* py_can_lire(PyObject *self, PyObject *args) {
     PyObject *error_msg;
     PyArrayObject *numpy_array;
     npy_intp dims[1],c[1];
     double *data;
     int i;
     if (handle.can_lire(&handle)==0) {
        fermer_can(); 
        error_msg = _get_error_msg();
        PyErr_SetObject(PyExc_ValueError,error_msg);
        return NULL;
     }
     dims[0] = handle.nombreEAdirect;
     numpy_array = (PyArrayObject*) PyArray_SimpleNew(1,dims,NPY_DOUBLE); 
     
     c[0] = 0;
     data = (double*)PyArray_GetPtr((PyArrayObject*)numpy_array,c);
     for (i=0; i<dims[0]; i++) data[i] = handle.tensionsLues[i];
     return Py_BuildValue("O",numpy_array);
}

static PyObject* py_can_config_sortie(PyObject *self, PyObject *args) {
    int nsortie,nbvaleurs,repetition;
    double te;
    PyObject *error_msg;
    PyObject *valeurs_array;
    double *valeurs;
    ARG_ERROR(!PyArg_ParseTuple(args,"idOi",&nsortie,&te,&valeurs_array,&repetition))
    if (NumpyList2Array((PyArrayObject*)valeurs_array,&valeurs,&nbvaleurs)==0) {
        fermer_can();
        PyErr_SetString(PyExc_ValueError,"erreur de format de l'echantillon");
        return NULL;     
    }   
    if (handle.can_config_sortie(&handle,nsortie,te,valeurs,nbvaleurs,repetition)==0) {
        fermer_can();
        error_msg = _get_error_msg();
        PyErr_SetObject(PyExc_ValueError,error_msg);
        return NULL;
    }
    Py_RETURN_NONE;
}

static PyObject* py_can_declencher_sorties(PyObject *self, PyObject *args) {
    int ns1,ns2;
    PyObject *error_msg;
    ARG_ERROR(!PyArg_ParseTuple(args,"ii",&ns1,&ns2))
    if (handle.can_declencher_sorties(&handle,ns1,ns2)==0) {
        fermer_can();
        error_msg = _get_error_msg();
        PyErr_SetObject(PyExc_ValueError,error_msg);
        return NULL;   
    }
    Py_RETURN_NONE;
}

static PyObject* py_can_acquerir_avec_sorties(PyObject *self, PyObject *args) {
     PyObject *valeurs1_array, *valeurs2_array;
     int nbvaleurs1,nbvaleurs2;
     double *valeurs1,*valeurs2;
     PyObject *error_msg;
     ARG_ERROR(!PyArg_ParseTuple(args,"OO",&valeurs1_array,&valeurs2_array))
     if (NumpyList2Array((PyArrayObject*)valeurs1_array,&valeurs1,&nbvaleurs1)==0) {
        fermer_can();
        PyErr_SetString(PyExc_ValueError,"erreur de format de l'echantillon 1");
        return NULL;     
     }
     if (NumpyList2Array((PyArrayObject*)valeurs2_array,&valeurs2,&nbvaleurs2)==0) {
         fermer_can();
         PyErr_SetString(PyExc_ValueError,"erreur de format de l'echantillon 2");
        return NULL;     
     }
     if (handle.can_acquerir_avec_sorties(&handle,valeurs1,nbvaleurs1,valeurs2,nbvaleurs2)==0) {
        fermer_can();
        error_msg = _get_error_msg();
        PyErr_SetObject(PyExc_ValueError,error_msg);
        return NULL;
     }
     Py_RETURN_NONE;
}

static PyObject* py_can_lancer_avec_sorties(PyObject *self, PyObject *args) {
     PyObject *valeurs1_array, *valeurs2_array;
     int nbvaleurs1,nbvaleurs2;
     double *valeurs1,*valeurs2;
     PyObject *error_msg;
     ARG_ERROR(!PyArg_ParseTuple(args,"OO",&valeurs1_array,&valeurs2_array))
     if (NumpyList2Array((PyArrayObject*)valeurs1_array,&valeurs1,&nbvaleurs1)==0) {
        fermer_can();
        PyErr_SetString(PyExc_ValueError,"erreur de format de l'echantillon 1");
        return NULL;     
     }
     if (NumpyList2Array((PyArrayObject*)valeurs2_array,&valeurs2,&nbvaleurs2)==0) {
        fermer_can();
         PyErr_SetString(PyExc_ValueError,"erreur de format de l'echantillon 2");
        return NULL;     
     }
     if (handle.can_lancer_avec_sorties(&handle,valeurs1,nbvaleurs1,valeurs2,nbvaleurs2)==0) {
        fermer_can();
        error_msg = _get_error_msg();
        PyErr_SetObject(PyExc_ValueError,error_msg);
        return NULL;
     }
     Py_RETURN_NONE;
}

static PyObject* py_can_stopper_sorties(PyObject *self, PyObject *args) {
    int ns1,ns2;
    PyObject *error_msg;
    ARG_ERROR(!PyArg_ParseTuple(args,"ii",&ns1,&ns2))
    SYSAM_PCI_NO
    if (handle.can_stopper_sorties(&handle,ns1,ns2)==0) {
        fermer_can();
        error_msg = _get_error_msg();
        PyErr_SetObject(PyExc_ValueError,error_msg);
        return NULL;   
    }
    Py_RETURN_NONE;
}

static PyObject* py_can_ecrire(PyObject *self, PyObject *args) {
     PyObject *error_msg;
     int ns1,ns2;
     double value1,value2;
     ARG_ERROR(!PyArg_ParseTuple(args,"idid",&ns1,&value1,&ns2,&value2))
     SYSAM_PCI_NO
     if (handle.can_ecrire(&handle,ns1,value1,ns2,value2)==0) {
         fermer_can();
         error_msg = _get_error_msg();
         PyErr_SetObject(PyExc_ValueError,error_msg);
         return NULL;  
     }
     Py_RETURN_NONE;
}

static PyObject* py_can_portC_config(PyObject *self, PyObject *args) {
     PyObject *error_msg;
     int bit,etat;
     ARG_ERROR(!PyArg_ParseTuple(args,"ii",&bit,&etat))
     SYSAM_PCI_NO
     if (handle.can_portC_config(&handle,bit,etat)==0) {
         fermer_can();
         error_msg = _get_error_msg();
         PyErr_SetObject(PyExc_ValueError,error_msg);
         return NULL;  
     }
     Py_RETURN_NONE; 
}
static PyObject* py_can_portC_ecrire(PyObject *self, PyObject *args) {
     PyObject *error_msg;
     int bit,etat;
     ARG_ERROR(!PyArg_ParseTuple(args,"ii",&bit,&etat))
     SYSAM_PCI_NO
     if (handle.can_portC_ecrire(&handle,bit,etat)==0) {
         fermer_can();
         error_msg = _get_error_msg();
         PyErr_SetObject(PyExc_ValueError,error_msg);
         return NULL;  
     }
     Py_RETURN_NONE; 
}
static PyObject* py_can_portC_lire(PyObject *self, PyObject *args) {
     PyObject *error_msg;
     int bit,bitC;
     ARG_ERROR(!PyArg_ParseTuple(args,"i",&bit))
     SYSAM_PCI_NO
     if (handle.can_portC_lire(&handle,bit)==0) {
         fermer_can();
         error_msg = _get_error_msg();
         PyErr_SetObject(PyExc_ValueError,error_msg);
         return NULL;  
     }
     bitC = handle.bitC;
     return Py_BuildValue("i",bitC);
}
static PyObject* py_can_portB_config(PyObject *self, PyObject *args) {
     PyObject *error_msg;
     int bit,etat;
     ARG_ERROR(!PyArg_ParseTuple(args,"ii",&bit,&etat))
     SYSAM_PCI_NO
     if (handle.can_portB_config(&handle,bit,etat)==0) {
         fermer_can();
         error_msg = _get_error_msg();
         PyErr_SetObject(PyExc_ValueError,error_msg);
         return NULL;  
     }
     Py_RETURN_NONE; 
}
static PyObject* py_can_portB_ecrire(PyObject *self, PyObject *args) {
     PyObject *error_msg;
     int bit,etat;
     ARG_ERROR(!PyArg_ParseTuple(args,"ii",&bit,&etat))
     SYSAM_PCI_NO
     if (handle.can_portB_ecrire(&handle,bit,etat)==0) {
         fermer_can();
         error_msg = _get_error_msg();
         PyErr_SetObject(PyExc_ValueError,error_msg);
         return NULL;  
     }
     Py_RETURN_NONE; 
}
static PyObject* py_can_portB_lire(PyObject *self, PyObject *args) {
     PyObject *error_msg;
     int bit,bitC;
     ARG_ERROR(!PyArg_ParseTuple(args,"i",&bit))
     SYSAM_PCI_NO
     if (handle.can_portB_lire(&handle,bit)==0) {
         fermer_can();
         error_msg = _get_error_msg();
         PyErr_SetObject(PyExc_ValueError,error_msg);
         return NULL;  
     }
     bitC = handle.bitC;
     return Py_BuildValue("i",bitC);
}

static PyObject* py_can_config_filtre(PyObject *self, PyObject *args) {
     PyObject *error_msg;
     PyObject *filtreA_array;
     PyObject *filtreB_array;
     double *A, *B;
     int nA,nB;
     ARG_ERROR(!PyArg_ParseTuple(args,"OO",&filtreA_array,&filtreB_array))
     SYSAM_PCI_NO
     if (NumpyList2Array((PyArrayObject*)filtreA_array,&A,&nA)==0) {
         fermer_can();
        PyErr_SetString(PyExc_ValueError,"erreur de definition du filtre");
        return NULL;
     }
     if (NumpyList2Array((PyArrayObject*)filtreB_array,&B,&nB)==0) {
        fermer_can();
        PyErr_SetString(PyExc_ValueError,"erreur de definition du filtre");
        return NULL;
     }
     if (handle.can_config_filtre(&handle,A,nA,B,nB)==0) {
        fermer_can();
        error_msg = _get_error_msg();
        PyErr_SetObject(PyExc_ValueError,error_msg);
        return NULL;
     }
     Py_RETURN_NONE;
}

static PyObject* py_can_config_compteur(PyObject *self, PyObject *args) {
    int entree,front_montant,front_descend;
    double hysteresis,duree;
    PyObject *error_msg;
    
    ARG_ERROR(!PyArg_ParseTuple(args,"iiidd",&entree,&front_montant,&front_descend,&hysteresis,&duree))
    SYSAM_PCI_NO
    if (handle.can_config_compteur(&handle,entree,front_montant,front_descend,hysteresis,duree)==0) {
        fermer_can();
        error_msg = _get_error_msg();
        PyErr_SetObject(PyExc_ValueError,error_msg);
        return NULL;
    }
    Py_RETURN_NONE;
}

static PyObject* py_can_compteur(PyObject *self, PyObject *args) {
   PyObject *error_msg;
   SYSAM_PCI_NO
   if (handle.can_compteur(&handle)==0) {
       fermer_can();
       error_msg = _get_error_msg();
       PyErr_SetObject(PyExc_ValueError,error_msg);
   }
   Py_RETURN_NONE;
}

static PyObject* py_can_lire_compteur(PyObject *self, PyObject *args) {
    PyObject *numpy_array;
    npy_intp dims[1], c[1];
    PyObject *error_msg;
    int *data;
    SYSAM_PCI_NO
     if (handle.can_lire_compteur(&handle)==0) {
        fermer_can();
        error_msg = _get_error_msg();
        PyErr_SetObject(PyExc_ValueError,error_msg);
        return NULL;
    }
    dims[0] = 2;
    numpy_array = PyArray_SimpleNew(1,dims,NPY_INT32);
    c[0] = 0;
    data = (int*)PyArray_GetPtr((PyArrayObject*)numpy_array,c);
    data[0] = handle.timer_high;
    data[1] = handle.timer_low;
    return Py_BuildValue("O",numpy_array);
}

static PyObject* py_can_config_chrono(PyObject *self, PyObject *args) {
    int entree,front_debut,front_fin;
    double hysteresis;
    PyObject *error_msg;
    ARG_ERROR(!PyArg_ParseTuple(args,"iiid",&entree,&front_debut,&front_fin,&hysteresis))
    SYSAM_PCI_NO
    if (handle.can_config_chrono(&handle,entree,front_debut,front_fin,hysteresis)==0) {
        fermer_can();
        error_msg = _get_error_msg();
        PyErr_SetObject(PyExc_ValueError,error_msg);
        return NULL;
    }
    Py_RETURN_NONE;
}

static PyObject* py_can_chrono(PyObject *self, PyObject *args) {
   PyObject *error_msg;
   SYSAM_PCI_NO
   if (handle.can_compteur(&handle)==0) {
       fermer_can();
       error_msg = _get_error_msg();
       PyErr_SetObject(PyExc_ValueError,error_msg);
   }
   Py_RETURN_NONE;
}

static PyObject* py_can_lire_chrono(PyObject *self, PyObject *args) {
    PyObject *numpy_array;
    npy_intp dims[1], c[1];
    PyObject *error_msg;
    int *data;
    SYSAM_PCI_NO
     if (handle.can_lire_chrono(&handle)==0) {
        fermer_can();
        error_msg = _get_error_msg();
        PyErr_SetObject(PyExc_ValueError,error_msg);
        return NULL;
    }
    dims[0] = 2;
    numpy_array = PyArray_SimpleNew(1,dims,NPY_UINT32);
    c[0] = 0;
    data = (int*)PyArray_GetPtr((PyArrayObject*)numpy_array,c);
    data[0] = handle.timer_high;
    data[1] = handle.timer_low;
    return Py_BuildValue("O",numpy_array);
}

static PyObject* py_can_afficher_calibrage(PyObject *self, PyObject *args) {
    SYSAM_PCI_NO
    handle.can_afficher_calibrage(&handle);
    Py_RETURN_NONE;
}

#ifdef AUDIO

int NumpyList2FloatArray(PyArrayObject *np_array, float **array, int *size) {
    int type,i;
    float v;
    void *data;
    type = PyArray_TYPE(np_array);
    if ((type!=NPY_DOUBLE)&&(type!=NPY_FLOAT)) return 0;
    if (PyArray_NDIM(np_array)!=1) return 0;
    *size = PyArray_DIM(np_array,0);
    *array = (float*)malloc(sizeof(float)*(*size));
    for (i=0; i<*size; i++) {
         data = PyArray_GETPTR1(np_array,i);
         switch(type) {
         case NPY_DOUBLE:
             v = (float)(((double*)data)[0]);
             (*array)[i] = v;
              break;
         case NPY_FLOAT:
             v = (float)(((float*)data)[0]);
             (*array)[i] = v;
             break;
         
         }
    }
    return 1;
}

float *table_ch1; 
float *table_ch2;
 
static PyObject* py_table_dds_start(PyObject *self, PyObject *args) {
    int sample_rate;
    int frames_per_buffer;
    double frequency_ch1,frequency_ch2;
    PyObject *array_ch1;
    PyObject *array_ch2;
    
    int size;
    ARG_ERROR(!PyArg_ParseTuple(args,"iiOOdd",&sample_rate,&frames_per_buffer,&array_ch1,&array_ch2,&frequency_ch1,&frequency_ch2))
    if (NumpyList2FloatArray((PyArrayObject*)array_ch1,&table_ch1,&size)==0) {
           PyErr_SetString(PyExc_ValueError,"table incorrecte");
           return NULL; 
    }
    if (size!=TABLE_SIZE) { 
           PyErr_SetString(PyExc_ValueError,"table de taille incorrecte");
           return NULL;
    }
    if (NumpyList2FloatArray((PyArrayObject*)array_ch2,&table_ch2,&size)==0) {
           PyErr_SetString(PyExc_ValueError,"table incorrecte");
           return NULL; 
    }
    if (size!=TABLE_SIZE) {
           PyErr_SetString(PyExc_ValueError,"table de taille incorrecte");
           return NULL;
    }
    if(table_dds_start(sample_rate,frames_per_buffer,table_ch1,table_ch2,frequency_ch1,frequency_ch2)>=0) {
        PyErr_SetString(PyExc_ValueError,"echec de lancement du flux audio");
        return NULL;   
    }
    
    Py_RETURN_NONE;
}

static PyObject* py_table_dds_stop(PyObject *self, PyObject *args) {
    int seconds;
    ARG_ERROR(!PyArg_ParseTuple(args,"i",&seconds))
    if (table_dds_stop(seconds)>=0) {
        PyErr_SetString(PyExc_ValueError,"echec de d'arret du flux audio");
        return NULL;   
    }
    free(table_ch1);
    free(table_ch2);
    Py_RETURN_NONE;
    
}

static PyObject* py_ring_buffer_init(PyObject *self, PyObject *args) {
    unsigned long number_blocks_exponent;
    unsigned long samples_per_block;
    int id;
    RingBuffer *buffer;
    ARG_ERROR(!PyArg_ParseTuple(args,"iii",&number_blocks_exponent,&samples_per_block,&id))
    buffer = (RingBuffer*) malloc(sizeof(RingBuffer));
    buffer->id = id;
    if (ring_buffer_init(buffer,number_blocks_exponent,samples_per_block)>0) {
         PyErr_SetString(PyExc_ValueError,"echec de creation du tampon");
         return NULL;   
    }
    return Py_BuildValue("i",buffer);
}

static PyObject* py_ring_buffer_delete(PyObject *self, PyObject *args) {
    int pointer;
    RingBuffer *buffer;
    ARG_ERROR(!PyArg_ParseTuple(args,"i",&pointer))
    buffer = (RingBuffer*) pointer;
    if (ring_buffer_delete(buffer)>0) {
          PyErr_SetString(PyExc_ValueError,"echec de destruction du tampon");
         return NULL; 
    }
    free(buffer);
    Py_RETURN_NONE;
}


static PyObject* py_ring_buffer_write(PyObject *self, PyObject *args) {
    int pointer;
    PyObject *array_data;
    RingBuffer *buffer;
    float *data;
    int size;
    ARG_ERROR(!PyArg_ParseTuple(args,"iO",&pointer,&array_data))
    buffer = (RingBuffer*) pointer;
    if (NumpyList2FloatArray((PyArrayObject*)array_data,&data,&size)==0) {
           PyErr_SetString(PyExc_ValueError,"tableau incorrect");
           return NULL; 
    }
    if (size!=buffer->samples_per_block) {
           PyErr_SetString(PyExc_ValueError,"tableau de taille incorrecte");
           return NULL;
    }
    
    if (ring_buffer_write(buffer,data)>0) {
           PyErr_SetString(PyExc_ValueError,"erreur d'ecriture dans le tampon");
           return NULL;
    }
    
    free(data);
    Py_RETURN_NONE;
}

static PyObject* py_output_stream_start(PyObject *self, PyObject *args) {
    int pointer_1, pointer_2;
    RingBuffer *buffer_1, *buffer_2;
    int sample_rate;
    ARG_ERROR(!PyArg_ParseTuple(args,"iii",&sample_rate,&pointer_1,&pointer_2))
    if (pointer_1!=0) {buffer_1 = (RingBuffer*) pointer_1;} else {buffer_1 = NULL;}
    if (pointer_2!=0) {buffer_2 = (RingBuffer*) pointer_2;} else {buffer_2 = NULL;}
    if (output_stream_start(sample_rate,buffer_1,buffer_2)>0) {
        PyErr_SetString(PyExc_ValueError,"erreur de lancement du flux audio");
        return NULL;
    }
    Py_RETURN_NONE;
}

static PyObject* py_output_stream_stop(PyObject *self, PyObject *args) {
    int seconds;
    ARG_ERROR(!PyArg_ParseTuple(args,"i",&seconds))
    if (output_stream_stop(seconds)>=0) {
        PyErr_SetString(PyExc_ValueError,"echec de d'arret du flux audio");
        return NULL;   
    }
    Py_RETURN_NONE;
}

#endif

static PyMethodDef pysysamMethods[] = {
    {"can_ouvrir",py_can_ouvrir,METH_VARARGS},
    {"can_fermer",py_can_fermer,METH_NOARGS},
    {"can_reset",py_can_reset,METH_NOARGS},
    {"can_config_entrees",py_can_config_entrees,METH_VARARGS},
    {"can_config_echantillon",py_can_config_echantillon,METH_VARARGS},
    {"can_config_quantification",py_can_config_quantification,METH_VARARGS},
    {"can_config_echantillon_permanent",py_can_config_echantillon_permanent,METH_VARARGS},
    {"can_config_trigger",py_can_config_trigger,METH_VARARGS},
    {"can_config_sortie",py_can_config_sortie,METH_VARARGS},
    {"can_acquerir",py_can_acquerir,METH_NOARGS},
    {"can_acquerir_permanent",py_can_acquerir_permanent,METH_NOARGS},
    {"can_lancer",py_can_lancer,METH_NOARGS},
    {"can_lancer_permanent",py_can_lancer_permanent,METH_VARARGS},
    {"can_stopper_acquisition",py_can_stopper_acquisition,METH_NOARGS},
    {"can_acquerir_avec_sorties",py_can_acquerir_avec_sorties,METH_VARARGS},
    {"can_lancer_avec_sorties",py_can_lancer_avec_sorties,METH_VARARGS},
    {"can_entrees",py_can_entrees,METH_VARARGS},
    {"can_entrees_filtrees",py_can_entrees_filtrees,METH_VARARGS},
    {"can_temps",py_can_temps,METH_VARARGS},
    {"can_nombre_echant",py_can_nombre_echant,METH_NOARGS},
    {"can_paquet",py_can_paquet,METH_VARARGS},
    {"can_paquet_filtrees",py_can_paquet_filtrees,METH_VARARGS},
    {"can_paquet_circulaire",py_can_paquet_circulaire,METH_VARARGS},
    {"can_paquet_circulaire_filtrees",py_can_paquet_circulaire_filtrees,METH_VARARGS},
    {"can_declencher_sorties",py_can_declencher_sorties,METH_VARARGS},
    {"can_stopper_sorties",py_can_stopper_sorties,METH_VARARGS},
    {"can_ecrire",py_can_ecrire,METH_VARARGS},
    {"can_activer_lecture",py_can_activer_lecture,METH_VARARGS},
    {"can_desactiver_lecture",py_can_desactiver_lecture,METH_NOARGS},
    {"can_lire",py_can_lire,METH_NOARGS},
    {"can_portC_config",py_can_portC_config,METH_VARARGS},
    {"can_portC_ecrire",py_can_portC_ecrire,METH_VARARGS},
    {"can_portC_lire",py_can_portC_lire,METH_VARARGS},
    {"can_portB_config",py_can_portB_config,METH_VARARGS},
    {"can_portB_ecrire",py_can_portB_ecrire,METH_VARARGS},
    {"can_portB_lire",py_can_portB_lire,METH_VARARGS},
    {"can_config_filtre",py_can_config_filtre,METH_VARARGS},
    {"can_config_compteur",py_can_config_compteur,METH_VARARGS},
    {"can_compteur",py_can_compteur,METH_NOARGS},
    {"can_lire_compteur",py_can_lire_compteur,METH_NOARGS},
    {"can_config_chrono",py_can_config_chrono,METH_VARARGS},
    {"can_chrono",py_can_chrono,METH_NOARGS},
    {"can_lire_chrono",py_can_lire_chrono,METH_NOARGS},
    {"can_afficher_calibrage",py_can_afficher_calibrage,METH_NOARGS},
#ifdef AUDIO
     {"table_dds_start",py_table_dds_start,METH_VARARGS},
     {"table_dds_stop",py_table_dds_stop,METH_VARARGS},
     {"ring_buffer_init",py_ring_buffer_init,METH_VARARGS},
     {"ring_buffer_delete",py_ring_buffer_delete,METH_VARARGS},
     {"ring_buffer_write",py_ring_buffer_write,METH_VARARGS},
     {"output_stream_start",py_output_stream_start,METH_VARARGS},
     {"output_stream_stop",py_output_stream_stop,METH_VARARGS},
#endif
    {NULL,NULL} 
};



#ifdef PYTHON3
static struct PyModuleDef pysysammodule = {
    PyModuleDef_HEAD_INIT,
    "pysysam",
    0,
    -1,
    pysysamMethods
};

PyMODINIT_FUNC
PyInit_pysysam(void) {
    import_array();
    return PyModule_Create(&pysysammodule);
    
}
#else
void initpysysam(void) {
    Py_InitModule("pysysam",pysysamMethods);
    import_array();   
}
#endif

